@@ -1,8 +1,8 @@
 #' Returns summary counts of occurrence records by one-degree cell for a single
 #' 		taxon, country, dataset, data publisher or data network.
-#' 
+#'
 #' @export
-occurrencedensity <- function() 
+occurrencedensity <- function()
 {
-	message("deprecated - now densitylist")
-}
\ No newline at end of file
+	message("Function deprecated. Now use densitylist()")
+}
