@@ -1,6 +1,6 @@
 #' Occurrencelist searches for taxon concept records matching a range of filters.
 #'
-#' @import RCurl XML plyr
+#' @import RCurl XML
 #' @param  scientificname scientitic name of taxon (character, see example)
 #' @param  taxonconceptKey unique key for taxon (numeric)
 #' @param  dataproviderkey Filter records to those provided by the supplied
@@ -72,7 +72,6 @@
 #'    (normalised) values used within the data portal (character)
 #' @param stylesheet Sets the URL of the stylesheet to be associated with the
 #     response document.
-#' @param latlongdf Return a data.frame of lat/long's for all occurrences (logical)
 #' @param removeZeros remove records with both Lat Long zero values (logical) 
 #' @param writecsv If path to a file is given, a text file is written out and 
 #' 		a success message is returned to the console (logical)
@@ -81,13 +80,20 @@
 #' @param curl If using in a loop, call getCurlHandle() first and pass
 #' the returned value in here (avoids unnecessary footprint)
 #' @examples \dontrun{
-#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 100, latlongdf = T)
-#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 100, latlongdf=F)
-#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 100, latlongdf=T, writecsv="~/myyyy.csv")
+#' # Query for a single species
+#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 100)
+#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 5)
 #' 
-#' # Many species
+#' # Query for many species, in this case using parallel fuctionality with plyr::llply
+#' library(doMC)
+#' registerDoMC(cores=4)
 #' splist <- c('Accipiter erythronemius', 'Junco hyemalis', 'Aix sponsa')
-#' out <- lapply(splist, function(x) occurrencelist(x, coordinatestatus = T, maxresults = 100, latlongdf = T))
+#' out <- llply(splist, function(x) occurrencelist(x, coordinatestatus = T, maxresults = 100), .parallel=F)
+#' lapply(out, head)
+#'
+#' # Write the output to csv file
+#' occurrencelist(scientificname = 'Accipiter erythronemius', coordinatestatus = TRUE, maxresults = 100, writecsv="~/myyyy.csv") 
+#' occurrencelist(scientificname = 'Erebia gorge*', coordinatestatus = TRUE, maxresults = 2000, writecsv="~/adsdf.csv")
 #' }
 #' @export
 occurrencelist <- function(scientificname = NULL, taxonconceptKey = NULL,
@@ -101,7 +107,7 @@ occurrencelist <- function(scientificname = NULL, taxonconceptKey = NULL,
 		originregioncode = NULL, startdate = NULL, enddate = NULL, startyear = NULL,
 		endyear = NULL, year = NULL, month = NULL, day = NULL, modifiedsince = NULL,
 		startindex = NULL, maxresults = 10, format = NA, icon = NULL,
-		mode = NULL, stylesheet = NULL, latlongdf = FALSE, removeZeros = FALSE, writecsv = NULL,
+		mode = NULL, stylesheet = NULL, removeZeros = FALSE, writecsv = NULL,
 		url = "http://data.gbif.org/ws/rest/occurrence/list", ..., curl = getCurlHandle()) 
 {	
 	args <- compact(
@@ -119,26 +125,54 @@ occurrencelist <- function(scientificname = NULL, taxonconceptKey = NULL,
 			originregioncode=originregioncode, startdate=startdate, enddate=enddate,
 			startyear=startyear, endyear=endyear, year=year, month=month, day=day,
 			modifiedsince=modifiedsince, startindex=startindex, format=format,
-			icon=icon, mode=mode, stylesheet=stylesheet, maxresults=maxresults
+			icon=icon, mode=mode, stylesheet=stylesheet, maxresults=as.integer(maxresults)
 		))
-	tt <- getForm(url, .params = args, curl = curl)
-	out <- xmlParse(tt)
-	if (latlongdf == TRUE) {
-		df <- gbifxmlToDataFrame(out, format)
+	counted <- getForm("http://data.gbif.org/ws/rest/occurrence/count", .params = args, curl = curl)
+	num <- as.numeric(xmlGetAttr(getNodeSet(xmlParse(counted), "//gbif:summary", namespaces="gbif")[[1]], "totalMatched"))
+	num <- min(c(num, maxresults))
+	if(num<=1000){
+		tt <- getForm(url, .params = args, curl = curl)
+		outlist <- xmlParse(tt)
+	} else
+		{
+			from <- seq(from=0, to=num, by=1000)
+			doit <- function(from) {
+				args$maxresults <- 1000
+				args$startindex <- from
+				tt <- getForm(url, .params = args, curl = curl)
+				xmlParse(tt)
+			}
+			outlist <- lapply(from, doit)
+		}
+	parseresults <- function(x) {
+		df <- gbifxmlToDataFrame(x, format)
 		df[, "decimalLongitude"] <- as.numeric(df[, "decimalLongitude"])
 		df[, "decimalLatitude"] <- as.numeric(df[, "decimalLatitude"])
 		i <- df[, "decimalLongitude"] == 0 & df[, "decimalLatitude"] == 0
 		if (removeZeros) {
 			df <- df[!i, ]
 		} else 
-			{
-				df[i, "decimalLatitude"] <- NA
-				df[i, "decimalLongitude"] <- NA 
-			}
+		{
+			df[i, "decimalLatitude"] <- NA
+			df[i, "decimalLongitude"] <- NA 
+		}
+		df		
+	}
+	if(length(outlist)==1){
+		dd <- parseresults(outlist)
 		if(!is.null(writecsv)){
-			write.csv(df, file=writecsv)
+			write.csv(dd, file=writecsv, row.names=F)
 			message("Success! CSV file written")
+		} else
+			{ dd }
+	} else 
+		{
+			outt <- lapply(outlist, parseresults)
+			dd <- do.call(rbind, outt)	
+			if(!is.null(writecsv)){
+				write.csv(dd, file=writecsv, row.names=F)
+				message("Success! CSV file written")
+			} else
+			 { dd }
 		}
-		df
-	} else { out }
 }
\ No newline at end of file
